"""
Names of notable colors because they are at the vertices of the RGB cube,
plus the fully-transparent one.
"""

from pytamaro.color_functions import rgb_color

black = rgb_color(0, 0, 0)
"""Black color

:meta hide-value:
"""

red = rgb_color(255, 0, 0)
"""Red color

:meta hide-value:
"""

green = rgb_color(0, 255, 0)
"""Green color

:meta hide-value:
"""

blue = rgb_color(0, 0, 255)
"""Blue color

:meta hide-value:
"""

yellow = rgb_color(255, 255, 0)
"""Yellow color

:meta hide-value:
"""

magenta = rgb_color(255, 0, 255)
"""Magenta color

:meta hide-value:
"""

cyan = rgb_color(0, 255, 255)
"""Cyan color

:meta hide-value:
"""

white = rgb_color(255, 255, 255)
"""White color

:meta hide-value:
"""

transparent = rgb_color(0, 0, 0, 0)
"""Fully-transparent color

:meta hide-value:
"""
