"""
Die folgenden nennenswerten Farben sind bereits definiert:
Farben an den Ecken des RGB Würfels, komplett durchsichtige Farbe.
"""

from __future__ import annotations

from pytamaro.color_names import (black, blue, cyan, green, magenta, red,
                                  transparent, white, yellow)
from pytamaro.de.color import Farbe

schwarz: Farbe = black
"""
Die Farbe Schwarz.

:meta hide-value:
"""

rot: Farbe = red
"""
Die Farbe Rot.

:meta hide-value:
"""

gruen: Farbe = green
"""
Die Farbe Grün.

:meta hide-value:
"""

blau: Farbe = blue
"""
Die Farbe Blau.

:meta hide-value:
"""

gelb: Farbe = yellow
"""
Die Farbe Gelb.

:meta hide-value:
"""

magenta: Farbe = magenta  # pylint: disable=self-assigning-variable
"""
Die Farbe Magenta.

:meta hide-value:
"""

cyan: Farbe = cyan  # pylint: disable=self-assigning-variable
"""
Die Farbe Cyan.

:meta hide-value:
"""

weiss: Farbe = white
"""
Die Farbe Weiss.

:meta hide-value:
"""

transparent: Farbe = transparent  # pylint: disable=self-assigning-variable
"""
Die komplett durchsichtige Farbe.

:meta hide-value:
"""
