"""
Nomi di colori notevoli essendo ai vertici del cubo RGB.
In aggiunta, il colore completamente trasparente.
"""

from __future__ import annotations

from pytamaro.color_names import black, blue, cyan, green, magenta, red, transparent, white, yellow
from pytamaro.it.color import Colore

nero: Colore = black
"""Colore nero

:meta hide-value:
"""

rosso: Colore = red
"""Colore rosso

:meta hide-value:
"""

verde: Colore = green
"""Colore verde

:meta hide-value:
"""

blu: Colore = blue
"""Colore blu

:meta hide-value:
"""

giallo: Colore = yellow
"""Colore giallo

:meta hide-value:
"""

magenta: Colore = magenta  # pylint: disable=self-assigning-variable
"""Colore magenta

:meta hide-value:
"""

ciano: Colore = cyan
"""Colore ciano

:meta hide-value:
"""

bianco: Colore = white
"""Colore bianco

:meta hide-value:
"""

trasparente: Colore = transparent
"""Colore completamente trasparente

:meta hide-value:
"""
