"""
(Re-)Export names for:
- basic types (Graphic and Color)
- color names
- operations
- primitves
- I/O
from this top-level module.
"""


from pytamaro.color import *
from pytamaro.color_functions import *
from pytamaro.color_names import *
from pytamaro.graphic import *
from pytamaro.io import *
from pytamaro.operations import *
from pytamaro.point import *
from pytamaro.point_names import *
from pytamaro.primitives import *

LANGUAGE = "en"

__version__ = "0.5.0"
